@@ -14,7 +14,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from __future__ import absolute_import
+import re
 
 from oslo_config import cfg
 from urllib import parse
@@ -45,6 +45,9 @@ CLIENT_ARGS = frozenset([
     'ssl_ca_certs',
     'sentinel',
     'sentinel_fallback',
+    'sentinel_username',
+    'sentinel_password',
+    'sentinel_ssl'
 ])
 """
 """
@@ -57,19 +60,20 @@ CLIENT_LIST_ARGS = frozenset([
 #: Client arguments that are expected to be boolean convertible.
 CLIENT_BOOL_ARGS = frozenset([
     'retry_on_timeout',
+    'socket_keepalive',
     'ssl',
+    'sentinel_ssl'
 ])
 
 #: Client arguments that are expected to be int convertible.
 CLIENT_INT_ARGS = frozenset([
     'db',
-    'health_check_interval',
-    'socket_keepalive',
 ])
 
 #: Client arguments that are expected to be float convertible.
 CLIENT_FLOAT_ARGS = frozenset([
     'socket_timeout',
+    'health_check_interval',
 ])
 
 OPTS = [
@@ -80,7 +84,7 @@ OPTS = [
 
   For example::
 
-    redis://[:password]@localhost:6379?db=0
+    redis://[username:password]@localhost:6379?db=0
 
   We proxy some options to the redis client (used to configure the redis client
   internals so that it works as you expect/want it to):  `%s`
@@ -115,6 +119,18 @@ OPTS = [
 ]
 
 
+def _parse_sentinel(sentinel):
+    # IPv6 (eg. [::1]:6379 )
+    match = re.search(r'^\[(\S+)\]:(\d+)$', sentinel)
+    if match:
+        return (match[1], int(match[2]))
+    # IPv4 or hostname (eg. 127.0.0.1:6379 or localhost:6379)
+    match = re.search(r'^(\S+):(\d+)$', sentinel)
+    if match:
+        return (match[1], int(match[2]))
+    raise ValueError('Malformed sentinel server format')
+
+
 def get_client(conf, scripts=None):
     if redis is None:
         raise RuntimeError("Redis Python module is unavailable")
@@ -130,6 +146,8 @@ def get_client(conf, scripts=None):
         if not parsed_url.path:
             raise ValueError("Expected socket path in parsed urls path")
         kwargs['unix_socket_path'] = parsed_url.path
+    if parsed_url.username:
+        kwargs['username'] = parsed_url.username
     if parsed_url.password:
         kwargs['password'] = parsed_url.password
 
@@ -152,20 +170,32 @@ def get_client(conf, scripts=None):
     # sentinel arg.
     if 'sentinel' in kwargs:
         sentinel_hosts = [
-            tuple(fallback.split(':'))
-            for fallback in kwargs.get('sentinel_fallback', [])
+            _parse_sentinel(fallback)
+            for fallback in kwargs.pop('sentinel_fallback', [])
         ]
-        sentinel_hosts.insert(0, (kwargs['host'], kwargs['port']))
+        sentinel_hosts.insert(0, (kwargs.pop('host'), kwargs.pop('port')))
+        sentinel_name = kwargs.pop('sentinel')
+        sentinel_kwargs = {}
+        # NOTE(tkajinam): Copy socket_* options, according to the logic
+        # in redis-py
+        for key in kwargs:
+            if key.startswith('socket_'):
+                sentinel_kwargs[key] = kwargs[key]
+        if kwargs.pop('sentinel_ssl', False):
+            sentinel_kwargs['ssl'] = True
+            for key in ('ssl_certfile', 'ssl_keyfile', 'ssl_cafile'):
+                if key in kwargs:
+                    sentinel_kwargs[key] = kwargs[key]
+        for key in ('username', 'password'):
+            if 'sentinel_' + key in kwargs:
+                sentinel_kwargs[key] = kwargs.pop('sentinel_' + key)
         sentinel_server = sentinel.Sentinel(
             sentinel_hosts,
-            socket_timeout=kwargs.get('socket_timeout'))
-        sentinel_name = kwargs['sentinel']
-        del kwargs['sentinel']
-        if 'sentinel_fallback' in kwargs:
-            del kwargs['sentinel_fallback']
+            sentinel_kwargs=sentinel_kwargs,
+            **kwargs)
         # The client is a redis.StrictRedis using a
         # Sentinel managed connection pool.
-        client = sentinel_server.master_for(sentinel_name, **kwargs)
+        client = sentinel_server.master_for(sentinel_name)
     else:
         client = redis.StrictRedis(**kwargs)
 
